/***********************************************************************
 * File: floating_point.c
 * 	(Extracted from
 * "$Id: floating_point.c,v 1.4 2007/01/27 19:04:03 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  	This C source implements the methods that compares two
 *	double/long double floating points, satisfactory.
 *
 * Copyright (C) 2005 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

/*
 * Refer URL:
 * "Comparing floating point numbers" by Bruce Dawson
 * http://www.cygnus-software.com/papers/comparingfloats/comparingfloats.htm
 */

/*
 * Compile switches:
 *	-DLITTLE_ENDIAN	: When the architecture is the "Little endian"
 *			  (ex. Intel x86), define this macro.
 *			  The default architecture is "big endian".
 *	-DUSE_STRTOLD	: When the C-compiler provides 'strtold()',
 *			  define this macro.
 *	-DLONG_DOUBLE	: When this macro is defined, it enables
 *			  'llo_xxx (Long Long Object)' functions,
 *			  defines 'LONG_' as the 8 bytes (long long)
 *			  integer and defines 'DOUBLE_' as the 12 bytes
 *			  (long double) floating points.
 *	-DNO_LONG_LONG	: When this macro is defined, it enables
 *			  'llo_xxx (Long Long Object)' functions,
 *			  defines 'LONG_' as the 4 bytes (long) integer
 *			  and defines 'DOUBLE_' as the 8 bytes (double)
 *			  floating points.
 *	-DUnitTest	: When this macro is defined, the API-s enables
 *			  verbose mode and the unit-test program is
 *			  enable.
 */

/*
 * include the definitions of this API
 */
#include "floating_point.h"


/*
 * Definitions of types, formatting and indexes for this method
 */
#if defined(LONG_DOUBLE) /* { */
	/*
	 * Definitions for 'long double'.
	 */
	#define DOUBLE_ long double	/* 12 bytes. floating point */
	#define DOUBLE_PRINTF_FMT_F "[%Le]"
	#define DOUBLE_PRINTF_FMT_E "[%Lf]"
	#define ULONG_	unsigned long	/* 4 * 3 bytes. unsigned */
	#define LONG_	long	  	/* 4 * 3 bytes. integers */
	#define LLO_CNT	LLO_LONG_DOUBLE_CNT /* long array max count */
#else /* } { !defined(LONG_DOUBLE) */
	/*
	 * Definitions for 'double'.
	 */
	#define DOUBLE_ double /* 8 bytes. floating point */
	#define DOUBLE_PRINTF_FMT_F "[%f]"
	#define DOUBLE_PRINTF_FMT_E "[%e]"
	#if defined(NO_LONG_LONG) /* { */
		/*
		 * for C-compilers unsupported the 'long long' 8bytes
		 * integers.
		 */
		#define ULONG_	unsigned long /* 8 bytes. unsigned */
		#define LONG_	long	      /* 8 bytes. integers */
	#endif /* } !defined(NO_LONG_LONG) */
#endif /* } !defined(LONG_DOUBLE) */



/*
 * When '-DUnitTest' is specified to compiler, this flag indicates
 * the verbose mode.  If this is set to non-zero, the API-s and helper
 * functions prints the result messages to 'stdout'.
 */
static int	vmode = 0;


/*
 * Defines the string representations of the IEEE754 special numbers
 * (not a number, infinite and negative infinite).
 */
#define SPECIAL_NUMBER_STR_BUFSZ 128
static char		not_a_number_str[SPECIAL_NUMBER_STR_BUFSZ];
static char		infinite_str[SPECIAL_NUMBER_STR_BUFSZ]
			, neg_infinite_str[SPECIAL_NUMBER_STR_BUFSZ];
static size_t	min_double_str_buf_sz;


/*
 * Declaration of the maximum unsigned long (or unsigned long long)
 * presentation significant.
 * 'max_ullps' can also be interpreted in terms of how many
 * representable floats we are willing to accept between A and B. 
 */
#if defined(LONG_DOUBLE) /* { */
	static LONG_ _max_ullps[LLO_CNT] = { -1L, -1L, -1L };
#elif defined(NO_LONG_LONG) /* { */
	static LONG_ _max_ullps[LLO_CNT] = { -1L, -1L };
#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	static long long _max_ullps = -1LL;
#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */


/*
 * Declarations of the zero and negative zero in IEEE754.
 * 'zero_as_int' and 'neg\zero_as_int' means integer representations.
 */
static DOUBLE_	zero, neg_zero;
#if (defined(NO_LONG_LONG) || defined(LONG_DOUBLE)) /* { */
	static LONG_	zero_as_int[LLO_CNT], neg_zero_as_int[LLO_CNT];
#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	static long long zero_as_int, neg_zero_as_int;
#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */


/*
 * Declarations of the not a numbers in IEEE754.
 * 'nan_as_int' means integer representations of 'the_nan'.
 */
static DOUBLE_	the_nan, nan_sqrt, nan_div_zero, nan_diff_inf
				, nan_add_1
				;
#if (defined(NO_LONG_LONG) || defined(LONG_DOUBLE)) /* { */
	static LONG_	nan_as_int[LLO_CNT];
#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	static long long	nan_as_int;
#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */


/*
 * Declarations of the infinite and negative infinite in IEEE754.
 * 'infinite_as_int' and 'neg_infinite_as_int' means integer
 * representations.
 */
static DOUBLE_	infinite, neg_infinite;
#if (defined(NO_LONG_LONG) || defined(LONG_DOUBLE)) /* { */
	static LONG_	infinite_as_int[LLO_CNT]
					, neg_infinite_as_int[LLO_CNT];
#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	static long long	infinite_as_int, neg_infinite_as_int;
#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */


/*
 * Declarations of the smallest number in IEEE754.
 * 'smallest_as_int' means integer representations.
 */
static DOUBLE_	smallest;
#if (defined(NO_LONG_LONG) || defined(LONG_DOUBLE)) /* { */
	static LONG_	smallest_as_int[LLO_CNT];
#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	static long long	smallest_as_int;
#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */


/*
 * Definitions to rename the API-s for 'long long double'.
 */
#if defined(LONG_DOUBLE) /* { */
	#define init_floating_point_lib	init_long_floating_point_lib
	#	define INIT_FLOATING_POINT_LIB			\
				"init_long_floating_point_lib"
	#define set_max_ullps		set_max_long_ullps
	#	define SET_MAX_ULLPS	"set_max_long_ullps"
	#define get_smallest		get_long_smallest
	#	define GET_SMALLEST	"get_long_smallest"
	#define is_not_a_number		is_not_a_long_number
	#	define IS_NOT_A_NUMBER	"is_not_a_long_number"
	#define is_infinite		is_long_infinite
	#	define IS_INFINITE	"is_long_infinite"
	#define is_negative		is_long_negative
	#	define IS_NEGATIVE	"is_long_negative"
	#define equals_doubles		equals_long_doubles
	#	define EQUALS_DOUBLES	"equals_long_doubles"
	#define cmp_double		cmp_long_double
	#	define CMP_DOUBLE	"cmp_long_double"
	#define get_min_size_of_double_string_buf		\
				get_min_size_of_long_double_string_buf
	#	define GET_MIN_SIZE_OF_DOUBLE_STRING_BUF	\
				"get_min_size_of_long_double_string_buf"
	#define string2double	string2long_double
	#	define STRING2DOUBLE	"string2long_double"
#else /* } { !defined(LONG_DOUBLE) */
	#	define INIT_FLOATING_POINT_LIB			\
				"init_floating_point_lib"
	#	define SET_MAX_ULLPS	"set_max_ullps"
	#	define GET_SMALLEST	"get_smallest"
	#	define IS_NOT_A_NUMBER	"is_not_a_number"
	#	define IS_INFINITE	"is_infinite"
	#	define IS_NEGATIVE	"is_negative"
	#	define EQUALS_DOUBLES	"equals_doubles"
	#	define CMP_DOUBLE	"cmp_double"
	#	define GET_MIN_SIZE_OF_DOUBLE_STRING_BUF	\
				"get_min_size_of_double_string_buf"
	#	define STRING2DOUBLE	"string2double"
#endif /* } defined(LONG_DOUBLE) */


/*
 * Declarations of zero, one, minimum and maximum as 'Long Long Object'.
 */
#if (defined(NO_LONG_LONG) || defined(LONG_DOUBLE)) /* { */
	static LONG_ llo_zero[LLO_CNT];	/* as like as 0LL */
	static LONG_ llo_one[LLO_CNT];	/* as like as 1LL */
	static LONG_ llo_minimum[LLO_CNT];
					/* as like as 0x8000...0000LL */
	static LONG_ llo_maximum[LLO_CNT];
					/* as like as 0x7fff...ffffLL */
#endif /* !(defined(NO_LONG_LONG) || defined(LONG_DOUBLE)) */


/*
 * This helper prints the 'Long Long Object'.
 */
static void print_llo(
	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
		LONG_ *llo
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
		long long llo
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	)
{
	#if defined(LONG_DOUBLE) /* { */
		if (llo[HIGH_LLO_IDX] != 0) {
			printf("(0x%lx%08lx%08lx)(%ld:%ld:%ld)"
			, llo[HIGH_LLO_IDX], llo[MID_LLO_IDX]
			,			llo[LOW_LLO_IDX]
			, llo[HIGH_LLO_IDX], llo[MID_LLO_IDX]
			,			llo[LOW_LLO_IDX]
			);
		}
		else if (llo[MID_LLO_IDX] != 0) {
			printf("(%lx%08lx)(:%ld:%ld)"
			, llo[MID_LLO_IDX], llo[LOW_LLO_IDX]
			, llo[MID_LLO_IDX], llo[LOW_LLO_IDX]
			);
		}
		else {
			printf("(%lx)(::%ld)"
			, llo[LOW_LLO_IDX]
			, llo[LOW_LLO_IDX]
			);
		}
	#elif defined(NO_LONG_LONG) /* } { */
		if (llo[HIGH_LLO_IDX] != 0) {
			printf(
			#if defined(LONG_DOUBLE) /* { */
				"(0x%llx%08llx)(%lld:%lld)"
			#else /* } { !defined(LONG_DOUBLE) */
				"(0x%lx%08lx)(%ld:%ld)"
			#endif /* } !defined(LONG_DOUBLE) */
			,	llo[HIGH_LLO_IDX], llo[LOW_LLO_IDX]
			,	llo[HIGH_LLO_IDX], llo[LOW_LLO_IDX]
			);
		}
		else {
			printf(
			#if defined(LONG_DOUBLE) /* { */
				"(0x%llx)(:%lld)"
			#else /* } { !defined(LONG_DOUBLE) */
				"(0x%lx)(:%ld)"
			#endif /* } !defined(LONG_DOUBLE) */
			,	llo[LOW_LLO_IDX], llo[LOW_LLO_IDX]
			);
		}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
		printf("(0x%llx)(%lld)", llo, llo);
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	return;
}


/*
 * This helper prints the 'double' or 'long double' number.
 */
static void print_num(DOUBLE_ d)
{
	printf(DOUBLE_PRINTF_FMT_F DOUBLE_PRINTF_FMT_E
	,			d,		d
	); print_llo(
	#if (defined(NO_LONG_LONG) || defined(LONG_DOUBLE)) /* { */
		(LONG_ *)(&d)
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
		AS_LONG_LONG(d)
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	);

	return;
}


/***********************************************************************
 * llo (means "long long object") functions definitions...
 **********************************************************************/
#if !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) /* { */
	/*
	 * definitions for the 'double' and compiler supporting
	 * 'long long'.
	 */
/*
 * "MAX" macro is defined for "long long".
 * "llo_max()" is defined for undef-ed "LONG_DOUBLE".
 * "llo_min()" is defined for "LONG_DOUBLE".
 */
#define MAX(A,B) (						\
			((A) > (B))? (A): (B)			\
		)

#else /* } { defined(NO_LONG_LONG) || defined(LONG_DOUBLE) */
	  /*
	   * Followings are declarations for the 'long double' or
	   * compiler unsupported 'long long' integer.
	   */
/*
 * This helper converts a 'double' (or 'long double') to the 'Long
 * Long Object'.
 */
static LONG_ *double2llo(LONG_ *llo, DOUBLE_ d)
{
	llo[HIGH_LLO_IDX]	= AS_HIGH_LONG(d);

	#if defined(LONG_DOUBLE) /* { for 'long double' */
		llo[MID_LLO_IDX]	= AS_MID_LONG(d);
	#endif /* } defined(LONG_DOUBLE) */

	llo[LOW_LLO_IDX]	= AS_LOW_LONG(d);

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ double2llo(%p ",		llo)
			;				print_num(d)
			; printf(") returns %p ",		llo)
			;				print_llo(llo)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	llo;
}


/*
 * This helper copies a 'Long Long Object' to another.
 */
static LONG_ *llo_copy(LONG_ *dest, LONG_ *src)
{
	dest[HIGH_LLO_IDX]	= src[HIGH_LLO_IDX];

	#if defined(LONG_DOUBLE) /* { for 'long double' */
		dest[MID_LLO_IDX]	= src[MID_LLO_IDX];
	#endif /* } defined(LONG_DOUBLE) */

	dest[LOW_LLO_IDX]	= src[LOW_LLO_IDX];

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_copy(%p, %p { ", dest,	src)
			;				print_llo(src)
			; printf("}) returns %p ",		dest)
			;				print_llo(dest)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	dest;
}


/*
 * This helper returns non-zero, when the 'Long Long Object' is
 * negative.  Otherwise, this returns zero.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'equals_long_doubles()' is enable.
 */
static int llo_is_negative(LONG_ *llo)
{
	int	result;

	#if defined(LONG_DOUBLE) /* { */
		/* for 'long double' */
		result	= (((llo[HIGH_LLO_IDX] & (1L << (sizeof(LONG_)
					* 4 - 1))) != 0)?
						!0: 0
				);
	#else /* } { !defined(LONG_DOUBLE) */
		/* for 'double' */
		result	= ((llo[HIGH_LLO_IDX] < 0)? !0: 0);
	#endif /* } !defined(LONG_DOUBLE) */

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_is_negative(%p {",	llo)
			;				print_llo(llo)
			; printf("}) returns %d.\n",		result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


#if defined(LONG_DOUBLE) /* { */
	/*
	 * declarations for compilers supporting 'long double'.
	 */
/*
 * This helper returns the least 'Long Long Object' of 'llo1' and
 * 'llo2'.
 */
static LONG_ *llo_min(LONG_ *llo1, LONG_ *llo2)
{
	LONG_	*result;

	/* 'llo1' and 'llo2' are integer representations of
	 * 'long double'.
	 */
	if (((((llo1[HIGH_LLO_IDX] == llo2[HIGH_LLO_IDX])
		&& llo1[MID_LLO_IDX] == llo2[MID_LLO_IDX])
		&& llo1[LOW_LLO_IDX] < llo2[LOW_LLO_IDX]
		)
	|| ((llo1[HIGH_LLO_IDX] == llo2[HIGH_LLO_IDX])
		&& llo1[MID_LLO_IDX] < llo2[MID_LLO_IDX])
		)
	|| llo1[HIGH_LLO_IDX] < llo2[HIGH_LLO_IDX]) {
		result	= llo1;
	}
	else {
		result	= llo2;
	}

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_min(%p {",		llo1)
			;				print_llo(llo1)
			; printf("}, %p {",			llo2)
			;				print_llo(llo2)
			; printf("}) returns %p.\n",		result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}
#else /* } { !defined(LONG_DOUBLE) */
	  /*
	   * declarations for 'double' or compilers unsupported
	   * 'long long'.
	   */
/*
 * This helper returns the maximum 'Long Long Object' of 'llo1' and
 * 'llo2'.
 */
static LONG_ *llo_max(LONG_ *llo1, LONG_ *llo2)
{
	LONG_	*result;

	/* 'llo1' and 'llo2' are integer representations of 'double'. */
	if (((llo1[HIGH_LLO_IDX] == llo2[HIGH_LLO_IDX])
		&& llo1[LOW_LLO_IDX] > llo2[LOW_LLO_IDX])
	|| llo1[HIGH_LLO_IDX] > llo2[HIGH_LLO_IDX]) {
		result	= llo1;
	}
	else {
		result	= llo2;
	}

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_max(%p {",		llo1)
			;				print_llo(llo1)
			; printf("}, %p {",			llo2)
			;				print_llo(llo2)
			; printf("}) returns %p.\n",		result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}
#endif /* } !defined(LONG_DOUBLE) */


/*
 * This helper returns the bit-inverted 'Long Long Object'.
 * The result of bit-inverting means the 1's complement.
 */
static LONG_ *llo_invert(LONG_ *result, LONG_ *llo)
{
	result[HIGH_LLO_IDX]	= ~((unsigned long)(llo[HIGH_LLO_IDX]));

	#if defined(LONG_DOUBLE) /* { */
		/* for 'long double' */
		result[MID_LLO_IDX] =
				~((unsigned long)(llo[MID_LLO_IDX]));
	#endif /* } defined(LONG_DOUBLE) */

	result[LOW_LLO_IDX]	= ~((unsigned long)(llo[LOW_LLO_IDX]));

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_invert(%p, %p {", result, llo)
			;			print_llo(llo)
			; printf("}) returns %p ",		result)
			;			print_llo(result)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns bit-anded 'Long Long Object'.
 */
static LONG_ *llo_and(LONG_ *result, LONG_ *llo1, LONG_ *llo2)
{
	result[HIGH_LLO_IDX] = llo1[HIGH_LLO_IDX] & llo2[HIGH_LLO_IDX];

	#if defined(LONG_DOUBLE) /* { */
		/* for 'long double' */
		result[MID_LLO_IDX] =
				llo1[MID_LLO_IDX] & llo2[MID_LLO_IDX];
	#endif /* } defined(LONG_DOUBLE) */

	result[LOW_LLO_IDX]	= llo1[LOW_LLO_IDX] & llo2[LOW_LLO_IDX];

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_and(%p, %p {",	result, llo1)
			;			print_llo(llo1)
			; printf("}, %p {",			llo2)
			;			print_llo(llo2)
			; printf("}) returns %p ",		result)
			;			print_llo(result)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns non-zero, when two 'Long Long Object'-s equals.
 * Otherwise, this returns zero.
 */
static int llo_eq(LONG_ *llo1, LONG_ *llo2)
{
	int	result;

	result = (llo1[HIGH_LLO_IDX] == llo2[HIGH_LLO_IDX]
		 #if defined(LONG_DOUBLE) /* { for 'long double' */
			&& llo1[MID_LLO_IDX] == llo2[MID_LLO_IDX]
		 #endif /* } defined(LONG_DOUBLE) */
		 && llo1[LOW_LLO_IDX] == llo2[LOW_LLO_IDX])? !0: 0;

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_eq(%p {",	llo1)
			;				print_llo(llo1)
			; printf("}, %p {",		llo2)
			;				print_llo(llo2)
			; printf("}) returns %d.\n",	result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns non-zero, when the 'Long Long Object' is zero.
 * Otherwise, this returns zero.
 */
static int llo_is_zero(LONG_ *llo)
{
	int	result	= llo_eq(llo_zero, llo);

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_is_zero(%p {",	llo)
			;				print_llo(llo)
			; printf("}) return %d.\n",	result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns non-zero, when the result of adding two
 * 'Long Long Object'-s occurs 'carry-over'.  Otherwise, this returns
 * zero.
 */
static int llo_is_carry_over_adding(ULONG_ ullo1, ULONG_ ullo2)
{
	int	result;
	ULONG_	diff;

	diff	= ULONG_MAX - ((ullo1 > ullo2)? ullo1: ullo2);
	result	= (diff < ((ullo1 < ullo2)? ullo1: ullo2));

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_is_carry_over_adding("
				  "(0x%lx)(%lu), (0x%lx)(%lu)"
				, ullo1, ullo1, ullo2, ullo2)
			; printf(") returns %d.\n", result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns the result of two positive 'Long Long Object'-s.
 */
static LONG_ *llo_add_simple(LONG_ *result, LONG_ *llo1, LONG_ *llo2)
{
	LONG_	high_llo1;
	int		is_carry_over;
	#if defined(LONG_DOUBLE) /* { */
		LONG_	mid_llo1;
	#endif /* } defined(LONG_DOUBLE) */

	high_llo1	= llo1[HIGH_LLO_IDX];
	is_carry_over	= llo_is_carry_over_adding(
					  (ULONG_)llo1[LOW_LLO_IDX]
					, (ULONG_)llo2[LOW_LLO_IDX]
					);
	result[LOW_LLO_IDX] = (LONG_)((ULONG_)llo1[LOW_LLO_IDX]
					+ (ULONG_)llo2[LOW_LLO_IDX])
					;

	#if defined(LONG_DOUBLE) /* { */
		/* for 'long double' */
		if (is_carry_over) {
			is_carry_over	=
				llo_is_carry_over_adding(
					  (ULONG_)llo1[MID_LLO_IDX]
					, 1U
					);
			if (is_carry_over) {
				/* add carry-bit from the mid digits to
				 * the high digits
				 */
				high_llo1	+= 1;
			}

			/* add carry-bit from the low digits to the mid
			 * digits
			 */
			mid_llo1	= llo1[MID_LLO_IDX] + 1;
		}
		else {
			mid_llo1	= llo1[MID_LLO_IDX];
		}
		is_carry_over = llo_is_carry_over_adding(
					  mid_llo1
					, (ULONG_)llo2[MID_LLO_IDX]
					);
		result[MID_LLO_IDX]	= mid_llo1 + llo2[MID_LLO_IDX];
	#endif /* } defined(LONG_DOUBLE) */

	if (is_carry_over) {
		/* add carry-bit from the low or mid digits */
		high_llo1	+= 1;
	}
	result[HIGH_LLO_IDX]	= high_llo1 + llo2[HIGH_LLO_IDX];

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_add_simple(%p, %p {"
			,			result, llo1)
			;			print_llo(llo1)
			; printf("}, %p {",		llo2)
			;			print_llo(llo2)
			; printf("}) returns %p ",	result)
			;			print_llo(result)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns the 2's complement of the 'Long Long Object'.
 */
static LONG_ *llo_twos_complement(LONG_ *result
				, LONG_ *ones_complement
				, LONG_ *llo
				)
{
	llo_add_simple(result
			, llo_one
			, llo_invert(ones_complement, llo)
			);

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_twos_complement(%p, %p, %p {"
			,		result, ones_complement, llo)
			;		print_llo(llo)
			; printf("}) returns %p ",		result)
			;		print_llo(result)
			; printf(" (output: %p ",	ones_complement)
			;		print_llo(ones_complement)
			; printf(").\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns the result of adding two 'Long Long Object'-s.
 */
static LONG_ *llo_add(LONG_ *result, LONG_ *llo1, LONG_ *llo2)
{
	int	negative1, negative2;

	if ((negative2 = llo_is_negative(llo2))
	&& (negative1 = llo_is_negative(llo1))) {
		/* Both of llo1 and llo2 is negative */
		LONG_	twos_complement_res1[LLO_CNT]
			, twos_complement_res2[LLO_CNT]
			, add_simple_res[LLO_CNT];

		llo_add_simple(add_simple_res
			, llo_twos_complement(twos_complement_res1
						, result
						, llo1
						)
			, llo_twos_complement(twos_complement_res2
						, result
						, llo2
						)
			);
		llo_twos_complement(result
				, twos_complement_res2
				, add_simple_res
				);
	}
	else {	/* llo1 and llo2 are positive. */
		llo_add_simple(result, llo1, llo2);
	}

	#if defined(LONG_DOUBLE) /* { */
		/* mask off the garbage high digits for 'long double' */
		result[HIGH_LLO_IDX]	&= 0x000000000000ffffL;
	#endif /* } defined(LONG_DOUBLE) */

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_add(%p, %p {",	result, llo1)
			;			print_llo(llo1)
			; printf("}, %p {",			llo2)
			;			print_llo(llo2)
			; printf("}) returns %p ",		result)
			;			print_llo(result)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns the result of subtracting of 'Long Long Object'.
 */
static LONG_ *llo_subtract(LONG_ *result, LONG_ *llo1, LONG_ *llo2)
{
	LONG_	invert_res[LLO_CNT], twos_complement[LLO_CNT];

	llo_add(result
		, llo1
		, llo_twos_complement(twos_complement, invert_res, llo2)
		);

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_subtract(%p, %p {", result, llo1
			);			print_llo(llo1)
			; printf("}, %p {",			llo2)
			;			print_llo(llo2)
			; printf("}) returns %p ",		result)
			;			print_llo(result)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns the absolute value of a 'Long Long Object'.
 */
static LONG_ *llo_abs(LONG_ *result, LONG_ *llo)
{
	if (llo_is_negative(llo)) {
		LONG_	invert_res[LLO_CNT];

		llo_add(result, llo_one, llo_invert(invert_res, llo));
	}
	else {
		llo_copy(result, llo);
	}

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_abs(%p, %p {",	result, llo)
			;			print_llo(llo)
			; printf("}) returns %p ",	result)
			;			print_llo(result)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This helper returns non-zero, when 'llo1' is least than 'llo2'.
 * Otherwise, this returns zero.
 */
static int llo_least(LONG_ *llo1, LONG_ *llo2)
{
	int	result;

	if (llo_eq(llo1, llo2)) {
		result	= 0; /* not least, but equals */
	}
	else {
		LONG_	abs_llo1[LLO_CNT], abs_llo2[LLO_CNT]
			, sub_res[LLO_CNT];

		if (llo_is_negative(llo1) && llo_is_negative(llo2)) {
			/* Both of two 'Long Long Object'-s is negative.
			 * Thus, make two absolute values from them
			 */
			llo_abs(abs_llo2, llo1)
			; llo_abs(abs_llo1, llo2)
			;
		}
		else {
			llo_copy(abs_llo1, llo1)
			; llo_copy(abs_llo2, llo2)
			;
		}

		result	= llo_is_negative(llo_subtract(sub_res
							, abs_llo1
							, abs_llo2
							)
						);
	}

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ llo_least(%p {",		llo1)
			;				print_llo(llo1)
			; printf("}, %p {",			llo2)
			;				print_llo(llo2)
			; printf("}) returns %d.\n",		result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}
	  /*
	   * Above are declarations for the 'long double' or compiler
	   * unsupported 'long long' integer.
	   */
#endif /* } defined(NO_LONG_LONG) || defined(LONG_DOUBLE) */


/***********************************************************************
 * definitions for "double" or "long double"
 **********************************************************************/

/*
 * This API returns the smallest number of 'double' or 'long double'.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'get_long_smallest()' is enable.
 */
DOUBLE_ get_smallest(void)
{
	DOUBLE_	result = smallest;

	/* initialize errno */
	errno	= 0;

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ get_smallest(void) returns ")
			;			print_num(smallest)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This API returns non-zero, when the number is 'not a number (NaN)'
 * in IEEE754.  Otherwise, this returns zero.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'equals_long_doubles()' is enable.
 */
int is_not_a_number(DOUBLE_ d)
{
	int 	result;

	/* initialize errno */
	errno	= 0;

	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' or compilers unsupported
		 * 'long long'
		 */
		LONG_	and_res[LLO_CNT];

		result	= llo_eq(llo_and(and_res
					, (LONG_ *)&d
					, nan_as_int
					)
				, nan_as_int
				);
	}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	{	/* for compilers supporting 'long long' */
		result = (((AS_LONG_LONG(d) & nan_as_int)
						== nan_as_int)? !0: 0
				);
	}
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ is_not_a_number("); print_num(d)
			; printf(") returns %d.\n",	result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This API returns non-zero, when the number is 'infinite' in IEEE754.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'equals_long_doubles()' is enable.
 */
int is_infinite(DOUBLE_ d)
{
	int	result;

	/* initialize errno */
	errno	= 0;

	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' or compilers unsupported
		 * 'long long'.
		 */
		LONG_	and_res[LLO_CNT];

		result	= llo_eq(llo_and(and_res
					, (LONG_ *)&d
					, nan_as_int
					)
				, infinite_as_int
				);
	}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	{	/* for compilers supporting 'long long' */
		result = (((AS_LONG_LONG(d) & nan_as_int)
					== infinite_as_int)? !0: 0
				);
	}
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ is_infinite("); print_num(d)
			; printf(") returns %d.\n",	result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This API returns non-zero, when the number is negative.
 * Otherwise, this returns zero.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * the API name becomes 'is_long_negative()'.
 */
int is_negative(DOUBLE_ d)
{
	int	result;

	/* initialize errno */
	errno	= 0;

	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' or compilers unsupported
		 * 'long long'
		 */
		LONG_	and_res[LLO_CNT];

		result	= ((llo_is_zero(llo_and(and_res
						, (LONG_ *)&d
						, neg_zero_as_int
						)
					)) == 0)? !0: 0;
	}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	{	/* for compilers supporting 'long long' */
		result	= (((AS_LONG_LONG(d) & neg_zero_as_int) != 0LL)?
					!0: 0
				);
	}
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ is_negative("); print_num(d)
			; printf(") returns %d.\n",	result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This API sets the maximum unsigned long (or unsigned long long)
 * presentation significant (max_ullps).
 * 'max_ullps' can also be interpreted in terms of how many
 * representable floats we are willing to accept between A and B. 
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'set_max_long_ullps()' is enable.
 *
 * 'max_ullps' typed as long pointer must be a pointer to long[LLO_CNT].
 *
 * If the argument isn't unsigned, this returns zero and sets 'errno'
 * to 'EINVAL (invalid argument)'.
 */
int set_max_ullps(
	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
		LONG_ *max_ullps
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
		long long max_ullps
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	)
{
	int	 result;

	/* initialize errno */
	errno	= 0;

	/* test arguments:
	 * if the 'max_ullps' is negative, it means invalid argument.
	 */
	if (
		#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
			llo_least(max_ullps, llo_zero)
		#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
			max_ullps < 0LL
		#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	) {	/* illegal 'max_ullps' argument */
		result	= errno = EINVAL; /* never set '_max_ullps' */
	}
	else {	/* set '_max_ullps' */
		#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
		{
			/* for 'long double' and compilers unsupported
			 * 'long long'.
			 */
			llo_copy(_max_ullps, max_ullps);
		}
		#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
		{	/* for compilers supporting 'long long' */
			_max_ullps	= max_ullps;
		}
		#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

		result	= 0; /* normal end */
	}

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ set_max_ullps(")
			;			print_llo(max_ullps)
			; printf(") returns %d. ", result)
			; printf("_max_ullps is set to ")
			;			print_llo(_max_ullps)
			; printf(".\n"); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This API initializes this package
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'init_long_floating_point_lib()' is enable.
 *
 * 'max_ullps' typed as long pointer must be a pointer to long[LLO_CNT].
 *
 * If the 'max_ullps' isn't unsigned, this returns zero and sets 'errno'
 * to 'EINVAL (invalid argument)'.
 */
int init_floating_point_lib(int verbose
	, 
	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
		LONG_ *max_ullps
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
		long long max_ullps
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	)
{
	/* initialize errno */
	errno	= 0;

	/* test arguments:
	 * the number is negative, it means invalid argument.
	 */
	if (set_max_ullps(max_ullps) != 0) {
		/* illegal 'max_ullps' argument */
		return errno; /* as EINVAL */
	}

	/* set verbose mode for the unit test */
	vmode	= verbose;

	/* initialize the 'Long Long Object' constants */
	#if defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' */
		llo_zero[HIGH_LLO_IDX]	= llo_zero[MID_LLO_IDX]
					= llo_zero[LOW_LLO_IDX] = 0L;
		llo_one[HIGH_LLO_IDX] 	= llo_one[MID_LLO_IDX]  = 0L;
		llo_one[LOW_LLO_IDX]  = 1L;
		llo_minimum[LOW_LLO_IDX] = llo_minimum[MID_LLO_IDX]
					 = 0L;
		llo_minimum[HIGH_LLO_IDX] = 0x8000L;
		llo_maximum[LOW_LLO_IDX] = llo_maximum[MID_LLO_IDX]
					 = 0xffffffff;
		llo_maximum[HIGH_LLO_IDX] = 0x7fffL;
	}
	#elif defined(NO_LONG_LONG) /* } { */
	{	/* for compilers unsupported 'long long' integer */
		llo_zero[LOW_LLO_IDX] = llo_zero[HIGH_LLO_IDX] = 0L;
		llo_one[LOW_LLO_IDX]  = 1L; llo_one[HIGH_LLO_IDX] = 0L;
		llo_minimum[LOW_LLO_IDX]	= 0L;
		llo_minimum[HIGH_LLO_IDX]	= LONG_MIN;
		llo_maximum[LOW_LLO_IDX]	= 0xffffffff;
		llo_maximum[HIGH_LLO_IDX]	= LONG_MAX;
	}
	#endif /* } defined(NO_LONG_LONG) */

	/* initialize the zero in IEEE754 */
	zero	= 0.0;
	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' or compilers unsupported
		 * 'long long'
		 */
		double2llo(zero_as_int, zero);
	}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	{	/* for 'compilers supporting 'long long' integer */
		zero_as_int = AS_LONG_LONG(zero);
	}
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	/* initialize the negative zero in IEEE754 */
	neg_zero	= -0.0;
	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' or compilers unsupported
		 * 'long long'
		 */
		double2llo(neg_zero_as_int, neg_zero);
	}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	{	/* for compilers supporting 'long long' integer */
		neg_zero_as_int = AS_LONG_LONG(neg_zero);
	}
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	/* initialize the infinite in IEEE754 */
	infinite	= 1.0 / zero;
	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' or compilers unsupported
		 * 'long long'
		 */
		double2llo(infinite_as_int, infinite);
	}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	{	/* for compilers supporting 'long long' */
		infinite_as_int	= AS_LONG_LONG(infinite);
	}
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	/* initialize the negative infinite in IEEE754 */
	neg_infinite	= -1.0 / zero;
	#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' or compilers unsupported
		 * 'long long'
		 */
		double2llo(neg_infinite_as_int, neg_infinite);
	}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	{	/* for compilers supporting 'long long' integer */
		neg_infinite_as_int	= AS_LONG_LONG(neg_infinite);
	}
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	/* initialize 'not a numbers' and the smallest number in
	 * IEEE754
	 */
	nan_div_zero	= zero / neg_zero;
	nan_sqrt	= sqrt(-1.0);
	nan_diff_inf = nan_add_1 = infinite - infinite;
	#if defined(LONG_DOUBLE) /* { */
	{	/* for 'long double' */
		AS_LOW_LONG(nan_add_1)	|= 1L;
		llo_copy((LONG_ *)&the_nan
			, llo_copy(nan_as_int
				, llo_min((LONG_ *)&nan_sqrt
					, llo_min((LONG_ *)&nan_div_zero
						, llo_min((LONG_ *)
							  &nan_diff_inf
							, (LONG_ *)
							  &nan_add_1
							)
						)
					)
				)
			);

		llo_copy((LONG_ *)&smallest, llo_one);
		llo_copy(smallest_as_int, llo_one);
	}
	#elif defined(NO_LONG_LONG) /* } { */
	{	/* for compilers unsupported 'long long' integer */
		AS_LOW_LONG(nan_add_1)	|= 1L;
		llo_copy((LONG_ *)&the_nan
			, llo_copy(nan_as_int
				, llo_max((LONG_ *)&nan_sqrt
					, llo_max((LONG_ *)&nan_div_zero
						, llo_max((LONG_ *)
							  &nan_diff_inf
							, (LONG_ *)
							  &nan_add_1
							)
						)
					)
				)
			);

		llo_copy((LONG_ *)&smallest, llo_one);
		llo_copy(smallest_as_int, llo_one);
	}
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	{	/* for compilers supporting 'long long' integer */
		AS_LONG_LONG(nan_add_1)	+= 1LL;
		AS_LONG_LONG(the_nan)
		= nan_as_int
		= MAX(AS_LONG_LONG(nan_sqrt)
			, MAX(AS_LONG_LONG(nan_div_zero)
				, MAX(AS_LONG_LONG(nan_diff_inf)
					, AS_LONG_LONG(nan_add_1)
					)
				)
			)
		;

		smallest = zero; AS_LONG_LONG(smallest) = 1LL;
		smallest_as_int	= AS_LONG_LONG(smallest);
	}
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	/* initialize the formatted string of 'not a number'
	 * in IEEE754
	 */
	(void)snprintf(not_a_number_str
		, sizeof(not_a_number_str)
		,
		#if defined(LONG_DOUBLE) /* { */
			"%Lf"	/* format for 'long long' integer */
		#else /* } { !defined(LONG_DOUBLE) */
			"%f"	/* format for 'long' integer */
		#endif /* } !defined(LONG_DOUBLE) */
		,	the_nan
		);

	/* initialize the formatted string of the infinite in IEEE754 */
	(void)snprintf(infinite_str
		, sizeof(infinite_str)
		,
		#if defined(LONG_DOUBLE) /* { */
			"%Lf"	/* format for 'long long' integer */
		#else /* } { !defined(LONG_DOUBLE) */
			"%f"	/* format for 'long' integer */
		#endif /* } !defined(LONG_DOUBLE) */
		,	infinite
		);

	/* initialize the formatted string of the negative infinite
	 * in IEEE754
	 */
	(void)snprintf(neg_infinite_str
		, sizeof(neg_infinite_str)
		,
		#if defined(LONG_DOUBLE) /* { */
			"%Lf"	/* format for 'long long' integer */
		#else /* } { !defined(LONG_DOUBLE) */
			"%f"	/* format for 'long' integer */
		#endif /* } !defined(LONG_DOUBLE) */
		,	neg_infinite
		);

	/* initialize the minimum size of the formatted string of
	 * 'double' or 'long double'
	 */
	min_double_str_buf_sz = 1 +
				((strlen(neg_infinite_str)
					> strlen(infinite_str))?
						strlen(neg_infinite_str)
				:		strlen(infinite_str))
				;

	#if defined(UnitTest) /* { */
		if (vmode) {
			#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
				printf("+++ llo_zero = ")
				;		print_llo(llo_zero)
				; printf(".\n"); (void)fflush(stdout);

				printf("+++ llo_one = ")
				;		print_llo(llo_one)
				; printf(".\n"); (void)fflush(stdout);

				printf("+++ llo_minimum = ")
				;		print_llo(llo_minimum)
				; printf(".\n"); (void)fflush(stdout);

				printf("+++ llo_maximum = ")
				;		print_llo(llo_maximum)
				; printf(".\n"); (void)fflush(stdout);
			#endif /* } defined(NO_LONG_LONG) || defined(LONG_DOUBLE) */

			printf("*** the_nan ");	print_num(the_nan)
			; printf(" == nan_sqrt "); print_num(nan_sqrt)
			; printf(" => %d.\n",	(the_nan == nan_sqrt))
			; (void)fflush(stdout);

			printf("*** is_not_a_number(the_nan) returns "
				"%d. ",	is_not_a_number(the_nan)
				); (void)fflush(stdout);

			printf("*** is_not_a_number(nan_sqrt) returns "
				  "%d",	is_not_a_number(nan_sqrt)
				); printf(".\n"); (void)fflush(stdout);

			printf("*** the_nan "); print_num(the_nan)
			; printf(" == nan_div_zero ")
			;		print_num(nan_div_zero)
			; printf(" => %d. ", (the_nan == nan_div_zero))
			; (void)fflush(stdout);

			printf("*** is_not_a_number(nan_div_zero) "
			); printf("returns %d.\n"
			,		is_not_a_number(nan_div_zero)
			); (void)fflush(stdout);

			printf("*** the_nan "); print_num(the_nan)
			; printf(" == nan_diff_inf ")
			; printf(" => %d. ", (the_nan == nan_diff_inf))
			; (void)fflush(stdout);

			printf("*** is_not_a_number(nan_diff_inf) ")
			; printf("returns %d.\n"
				,	is_not_a_number(nan_diff_inf)
				); (void)fflush(stdout);

			printf("*** the_nan "); print_num(the_nan)
			; printf(" == nan_add_1 "); print_num(nan_add_1)
			; printf(" => %d. ",	(the_nan == nan_add_1))
			; (void)fflush(stdout);

			printf("*** is_not_a_number(nan_add_1) ")
			; printf("returns %d.\n"
				, is_not_a_number(nan_add_1)
				); (void)fflush(stdout);

			printf("*** zero "); print_num(zero)
			; printf(" == neg_zero "); print_num(neg_zero)
			; printf(" => %d.\n",	(zero == neg_zero))
			; (void)fflush(stdout);

			printf("*** smallest = "); print_num(smallest)
			; printf(".\n"); (void)fflush(stdout);

			printf("*** \"%%f\" and \"%%e\" to infinite "
				"outputs ")
			;		print_num(infinite)
			; printf(".  infinite_as_int is ")
			;		print_llo(infinite_as_int)
			; printf(" : "
				#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
					"llo_maximum = "
					);	print_llo(llo_maximum
				#else /* } { !defined(NO_LONG_LONG)
				       *	&& !defined(LONG_DOUBLE)
				       */
					"LONG_LONG_MAX = "
					);	print_llo(LONG_LONG_MAX
				#endif /* } !defined(NO_LONG_LONG)
					*	&& !defined(LONG_DOUBLE)
					*/
				)
			; printf(".\n"); (void)fflush(stdout);

			printf("*** \"%%f\" and \"%%e\" to "
				"neg_infinite outputs "
			);		print_num(neg_infinite)
			; printf(".  neg_infinite_as_int = ")
			;		print_llo(neg_infinite_as_int)
			; printf(" : "
				#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
					"llo_minimum = "
					);	print_llo(llo_minimum
				#else /* } { !defined(NO_LONG_LONG)
				       *	&& !defined(LONG_DOUBLE)
				       */
					"LONG_LONG_MIN = "
					);	print_llo(LONG_LONG_MIN
				#endif /* } !defined(NO_LONG_LONG)
					*	&& !defined(LONG_DOUBLE)
					*/
				)
			; printf(".\n"); (void)fflush(stdout);

			printf("*** min_double_str_buf_sz = %lu. "
				"infinite_str = \"%s\". "
				"neg_infinite_str = \"%s\".\n"
				, (unsigned long)min_double_str_buf_sz
				, infinite_str
				, neg_infinite_str
				); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	0; /* normal end */
}


/*
 * This helper returns the result of comparing number and infinite.
 *	d1 is negative infinite, d2 is negative infinite => returns 0.
 *	d1 is negative infinite, d2 isn't infinite	 => returns -1.
 *	d1 is positive infinite, d2 is positive infinite => returns 0.
 *	d1 is positive infinite, d2 isn't infinite	 => returns 1.
 */
static int cmp_with_infinite(DOUBLE_ d1, DOUBLE_ d2)
{
	int	result = -2;

	if (is_infinite(d1) != 0) {
		if (is_negative(d1) != 0) {
			result	= ((((is_infinite(d2) != 0)
					&& is_negative(d2) != 0) != 0)?
						0: -1
					);
		}
		else {
			result	= ((((is_infinite(d2) != 0)
					&& is_negative(d2) == 0) != 0)?
						0: 1
					);
		}
	}
	else if (is_infinite(d2) != 0) {
		if (is_negative(d2) != 0) {
			result	= ((((is_infinite(d1) != 0)
					&& is_negative(d1) != 0) != 0)?
						0: 1
					);
		}
		else {
			result	= ((((is_infinite(d1) != 0)
					&& is_negative(d1) == 0) != 0)?
						0: -1
					);
		}
	}

	return	result;
}


/*
 * This API returns non-zero, when the numbers equals.
 * Otherwise, this returns zero.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'equals_long_doubles()' is enable.
 *
 * If the 'max_ullps' is negative, it means that this API wasn't
 * initialized, yet.  This returns 0 and set 'errno' to 'EINVAL'.
 * If the number is 'not a number', this returns zero and set 'errno'
 * to 'ERANGE'.
 */
int equals_doubles(DOUBLE_ d1, DOUBLE_ d2)
{
	int	result;

	/* initialize errno as zero (no error) */
	errno	= 0;

	/* test: this API-s wasn't initialized? */
	if (
		#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
			llo_is_negative(_max_ullps)
		#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
			_max_ullps < 0LL
		#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	) {
		/* API wasn't initialized.  occurs 'invalid argument'
		 * error
		 */
		result	= 0; /* always, false */
		errno	= EINVAL; /* library wasn't initialized, yet */
	}
	else if (is_not_a_number(d1) != 0 || is_not_a_number(d2) != 0) {
		/* to compare NaN and double means nothing. occurs
		 * 'out of range' error
		 */
		result	= 0; /* always, false */
		errno	= ERANGE; /* out of range */
	}
	else if (is_infinite(d1) != 0 || is_infinite(d2) != 0) {
		/* compare infinite and double */
		result = (cmp_with_infinite(d1, d2) == 0)? !0: 0;
	}
	else if ((result = is_negative(d1)) != is_negative(d2)) {
		/* signs of d1 and d2 are not the same. */
		result	= (d1 == d2)? !0: 0;
	}
	else {	/* compare the usual numbers */
		/* test: the numbers are negatives? */
		if (result != 0) {
			/* Both of d1 and d2 are negatives */
			#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
			{	/* for 'long double' or compilers
				 * unsupported 'long long'
				 */
				LONG_	sub_res[LLO_CNT];

				llo_copy((LONG_ *)&d1
					, llo_subtract(sub_res
						, neg_zero_as_int
						, (LONG_ *)&d1
						)
					);
				llo_copy((LONG_ *)&d2
					, llo_subtract(sub_res
						, neg_zero_as_int
						, (LONG_ *)&d2
						)
					);
			}
			#else /* } { !defined(NO_LONG_LONG)
			       *	&& !defined(LONG_DOUBLE)
			       */
			{	/* for compilers supporting 'long long'
				 */
				AS_LONG_LONG(d1)
				= neg_zero_as_int - AS_LONG_LONG(d1)
				;
				AS_LONG_LONG(d2)
				= neg_zero_as_int - AS_LONG_LONG(d2)
				;
			}
			#endif /* } !defined(NO_LONG_LONG)
				*	&& !defined(LONG_DOUBLE)
				*/
		}

		/* If the differ of the numbers is equals to 'max_ullps'
		 * or the differ is least than 'max_ullps', the result
		 * of this API is non-zero.  Otherwise, the result is
		 * zero.
		 */
		#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
		{	/* for 'long double' or compilers unsupported
			 * 'long long'
			 */
			LONG_	sub_res[LLO_CNT], abs_res[LLO_CNT];

			llo_abs(abs_res
				, llo_subtract(sub_res
						, (LONG_ *)&d1
						, (LONG_ *)&d2
						)
				);
			result	= (	(llo_eq(abs_res, _max_ullps)
					|| llo_least(abs_res
						, _max_ullps
						)
					)?		!0: 0
					);
		}
		#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
			result	= (	(llabs((long long)
						(AS_LONG_LONG(d1)
						- AS_LONG_LONG(d2))
						) <= _max_ullps)?
							!0: 0
					);
		#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
	}

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ equals_doubles(");	print_num(d1)
			; printf(" , ");		print_num(d2)
			; printf(") returns %d.\n",	result)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This API returns the result of comparing two numbers.
 * If d1 equals d2, this returns zero.
 * If d1 is least than d2, this returns the negative.
 * Otherwise (d1 is greater than d2), this returns positive.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'equals_long_doubles()' is enable.
 *
 * If the 'max_ullps' is negative, it means that this API wasn't
 * initialized, yet.  This returns 0 and set 'errno' to 'EINVAL'.
 * If the number is 'not a number', this returns zero and set 'errno'
 * to 'ERANGE'.
 */
int cmp_double(DOUBLE_ d1, DOUBLE_ d2)
{
	int	result;

	/* initialize errno as zero (no error) */
	errno	= 0;

	/* test: 'd1' or 'd2' is infinite? */
	if (((is_not_a_number(d1) == 0)
	&& is_not_a_number(d2) == 0)
	&& ((is_infinite(d1) != 0) || is_infinite(d2) != 0)
	) {	/* invoke "comparing with infinite" helper */
		result	= cmp_with_infinite(d1, d2);
	}
	else { /* compare two numbers */
		result = (equals_doubles(d1, d2) != 0)?	0
			 : ((d1 < d2)?			-1
			 :				1
			 );
	}

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ cmp_double(");	print_num(d1)
			; printf(", ");			print_num(d2)
			; printf(") returns %d.\n",	result
			); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This API returns the minimum size of the formatted string for
 * 'double' or 'long double'.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'equals_long_doubles()' is enable.
 */
size_t get_min_size_of_double_string_buf(void)
{
	size_t	result = min_double_str_buf_sz;
	
	/* initialize errno */
	errno	= 0;

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ get_min_size_of_double_buf_size("
				"void) returns %lu.\n"
				,		(unsigned long)result
				); (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


/*
 * This API returns the 'double' or 'long double' that is converted
 * a string.
 *
 * If '-DLONG_DOUBLE' is specified to the compiler,
 * 'equals_long_doubles()' is enable.
 *
 * If the string is NULL, this returns negative infinite and set 'errno'
 * to 'EINVAL'.
 * If the string is 'not a number' in IEEE754, this returns NaN and
 * set 'errno' to 'ERANGE'.
 * If failed to convert the string, this returns zero or negative zero
 * and set 'errno' to 'ERANGE'.
 */
DOUBLE_ string2double(const char *s)
{
	char	*endp = (char *)s;
	DOUBLE_	result;

	/* initialize errno */
	errno	= 0;

	/* test: argument is invalid? */
	if (s == NULL) {
		result	= neg_infinite;	/* returns negative infinite */
		errno	= EINVAL; /* illegal argument */

		return	result; /* invalid arguments */
	}

	/* test: the string equals the special numbers strings? */
	if (strcmp(not_a_number_str, s) == 0) {
		/* the string matches 'not a number' string */
		result	= the_nan;	/* returns 'not a number' */
		errno	= ERANGE; /* out of range */
	}
	else if (strcmp(neg_infinite_str, s) == 0) {
		/* the string matches 'negative infinite' string */
		result	= neg_infinite;	/* returns negative infinite */
	}
	else if (strcmp(infinite_str, s) == 0) {
		/* the string matches 'infinite' string */
		result	= infinite;	/* returns infinite */
	}
	else { /* the string is a usual string */
		/* convert the string to 'double' */
		#if (defined(LONG_DOUBLE) && defined(USE_STRTOLD)) /* { */
			#define CONV	strtold	/* for 'long double' */
		#else /* } { !(defined(LONG_DOUBLE) && defined(USE_STRTOLD)) */
			#define CONV	strtod	/* for 'double' */
		#endif /* } !(defined(LONG_DOUBLE) && defined(USE_STRTOLD)) */
		errno = 0; result = CONV(s, &endp);
		#undef	CONV

		/* test: failed to convert the string and the number is
		 * zero?
		 */
		if ((errno == ERANGE) && result == 0.0) {
			/* out of range:
			 * the string is non-numeric, empty or
			 * infinitesimal (-0.0, 0.0)
			 */
			for (endp = (char *)s
			; (*endp != '\0') && isspace(*endp)
			; ++endp) { ; /* skip space codes */ }

			/* test: the first character is negative sign
			 * code?
			 */
			if (*endp == '-') {
				/* means negative infinitesimal -0.0 */
				result	= neg_zero;
			}
			else { /* non-numeric, empty or positive
				* infinitesimal:
				* it means positive infinitesimal 0.0
				*/
				result	= zero;
			}
		}
	}

	#if defined(UnitTest) /* { */
		if (vmode) {
			printf("+++ string2double(\"%s\") returns ", s)
			;			print_num(result)
			; printf(".  (errno = %d)\n",	errno)
			; (void)fflush(stdout);
		}
	#endif /* } defined(UnitTest) */

	return	result;
}


#undef MAX /* } used in this library, only */

/*---------- above is definitions in source of library --------*/

#if defined(UnitTest) /* { */
/***********************************************************************
 * Unit Test program
 **********************************************************************/
static char *chop(char *s)
{
	char	*cp = s + (strlen(s) - 1);

	if (*cp == '\n') {
		*cp	= '\0';
	}
	if (*(cp = s + (strlen(s) - 1)) == '\r') {
		*cp	= '\0';
	}

	return	s;
}


int main(int ac, char **av)
{
	int	verbose;
	char		buf1[4096], buf2[4096], *endp;
	DOUBLE_		d1, d2;
	int		result;
	#if defined(LONG_DOUBLE) /* { */
		LONG_	i, j, max_ullps[LLO_LONG_DOUBLE_CNT]
			, llo_nega_one[LLO_LONG_DOUBLE_CNT]
			, llo1[5][LLO_LONG_DOUBLE_CNT]
			, llo2[5][LLO_LONG_DOUBLE_CNT]
			, llo_res[LLO_LONG_DOUBLE_CNT]
			, llo_res1[LLO_LONG_DOUBLE_CNT];
	#elif defined(NO_LONG_LONG) /* } { */
		LONG_	i, j, max_ullps[LLO_CNT], llo_nega_one[LLO_CNT]
			, llo1[5][LLO_CNT], llo2[5][LLO_CNT]
			, llo_res[LLO_CNT], llo_res1[LLO_CNT];
	#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
		long long i, max_ullps = DEFAULT_max_ullps;
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	#if defined(LONG_DOUBLE) /* { */
		max_ullps[HIGH_LLO_IDX] = max_ullps[MID_LLO_IDX] = 0L;
		max_ullps[LOW_LLO_IDX]	= 10L;
	#elif defined(NO_LONG_LONG) /* } { */
		max_ullps[HIGH_LLO_IDX] = 0L;
		max_ullps[LOW_LLO_IDX]	= 10L;
	#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

	if (ac > 1) {
		#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
			ULONG_ mullps = strtoul(av[1], &endp, 0);
		#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
			long long mullps = strtoll(av[1], &endp, 0);
		#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

		if ((errno == ERANGE)
		&& (
			#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
				mullps == ULONG_MAX)
			#else /* } { !defined(NO_LONG_LONG)
			       *	&& !defined(LONG_DOUBLE)
			       */
				(mullps == LONG_LONG_MIN)
				|| mullps == LONG_LONG_MAX)
			#endif /* } !defined(NO_LONG_LONG)
				*	&& !defined(LONG_DOUBLE)
				*/
		) {
			fprintf(stderr
				, "Usage: %s [max_ullps]\n"
				  "    max_ullps : non-negative "
				  "and small enough that the "
				  "default NAN won't compare as "
				  "equal to anything.\n"
				,	av[0]
				); (void)fflush(stderr);
			exit(1); /* usage error */
		}

		#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
			max_ullps[LOW_LLO_IDX]	= mullps;
		#else /* } { !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */
			max_ullps	= mullps;
		#endif /* } !defined(NO_LONG_LONG) && !defined(LONG_DOUBLE) */

		verbose = 1;
	}
	else {
		verbose = 0;
	}

	/* initialize "floating point" library */
	printf(">>> UnitTest: %s(%d, ", INIT_FLOATING_POINT_LIB, verbose
	);				print_llo(max_ullps)
	; printf(")...\n"); (void)fflush(stdout);

	result = init_floating_point_lib(verbose, max_ullps);
						/* verbose mode */
	printf("<<< UnitTest: %s(%d, ",	INIT_FLOATING_POINT_LIB, verbose
	);				print_llo(max_ullps)
	; printf(") returns %d!\n\n", result)
	; (void)fflush(stdout);
	if (result != 0) {
		printf("***FATAL:%s:L%d: %s() failed.\n"
			,	__FILE__, __LINE__
			,		INIT_FLOATING_POINT_LIB
			); (void)fflush(stdout);

		exit(2); /* fatal error */
	}

	/* test 'long long object' */
	#if (defined(LONG_DOUBLE) || defined(NO_LONG_LONG)) /* { */
	{	printf(">>> UnitTest: 'long long objects'...\n")
		; (void)fflush(stdout);

		llo_copy(llo1[0], llo_minimum);
		llo_copy(llo2[0], llo_minimum);
		llo_copy(llo1[1], llo_twos_complement(llo_nega_one
						, llo_res1
						, llo_one
						));
		llo_copy(llo2[1], llo_twos_complement(llo_nega_one
						, llo_res1
						, llo_one
						));
		llo_copy(llo1[2], llo_zero);
		llo_copy(llo2[2], llo_zero);
		llo_copy(llo1[3], llo_one);
		llo_copy(llo2[3], llo_one);
		llo_copy(llo1[4], llo_maximum);
		llo_copy(llo2[4], llo_maximum);

		for (i = 0; i < (sizeof(llo1) / sizeof(llo1[0])); ++i)
		for (j = 0; j < (sizeof(llo1) / sizeof(llo1[0])); ++j) {
			llo_abs(llo_res, llo1[j]);
			printf("*** llo_abs(%p, %p[%ld] {"
				,	llo_res, llo1[j], j)
			;			print_llo(llo1[j])
			; printf("}) returns %p {", llo_res)
			;			print_llo(llo_res)
			; printf("}.\n"); (void)fflush(stdout);

			llo_twos_complement(llo_res, llo_res1, llo1[j]);
			printf("*** llo_twos_complement(%p, %p, %p[%ld]"
				" {"
				,	llo_res, llo_res1, llo1[j], j)
			;			print_llo(llo1[j])
			; printf("}) returns %p {", llo_res)
			;			print_llo(llo_res)
			; printf("} (outputs: %p {", llo_res1)
			;			print_llo(llo_res1)
			; printf(".\n"); (void)fflush(stdout);

			llo_add(llo_res, llo1[i], llo2[j]);
			printf("*** llo_add(%p, %p[%ld] {"
				,		llo_res, llo1[i], i
				)
			;			print_llo(llo1[i])
			; printf("}, %p[%ld] {", llo2[j], j)
			;			print_llo(llo2[j])
			; printf("}) returns %p {", llo_res)
			;			print_llo(llo_res)
			; printf("}.\n"); (void)fflush(stdout);

			llo_subtract(llo_res, llo1[i], llo2[j]);
			printf("*** llo_subtract(%p, %p[%ld] {"
				,		llo_res, llo1[i], i)
			;			print_llo(llo1[i])
			; printf("}, %p[%ld] {", llo2[j], j)
			;			print_llo(llo2[j])
			; printf("}) returns %p {", llo_res)
			;			print_llo(llo_res)
			; printf("}.\n"); (void)fflush(stdout);
		}

		printf("<<< UnitTest: 'long long objects' finished!\n\n"
		); (void)fflush(stdout);
	}
	#endif /* } !(defined(LONG_DOUBLE) || defined(NO_LONG_LONG)) */

	/* test equals_doubles() */
	{	printf(">>> UnitTest: %s() and 'max_ullps = "
			,	EQUALS_DOUBLES
			);			print_llo(max_ullps)
		; printf("'...\n")
		; (void)fflush(stdout);
		for (i = 0
			, d2 = get_smallest()
			, d1 = get_smallest()
			, result = 1
		; (result != 0)
		  &&
			#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
				i <= 10 * max_ullps[0] + 1
			#else /* } { !defined(NO_LONG_LONG)
			       *	&& !defined(LONG_DOUBLE)
			       */
				i <= 10LL * max_ullps + 1LL
			#endif /* } !defined(NO_LONG_LONG)
				*	&& !defined(LONG_DOUBLE)
				*/
		; ++i
		,
			#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
				*((LONG_ *)&d2)	+= *((LONG_ *)&d1)
			#else /* } { !defined(NO_LONG_LONG)
			       *	&& !defined(LONG_DOUBLE)
			       */
				AS_LONG_LONG(d2) += AS_LONG_LONG(d1)
			#endif /* } !defined(NO_LONG_LONG)
				*	&& !defined(LONG_DOUBLE)
				*/
		) {
			printf("***[%d] %s("
				#if defined(LONG_DOUBLE) /* { */
					"[%.100Lf][%Le]"
				#else /* } { !defined(LONG_DOUBLE) */
					"[%.100f][%e]"
				#endif /* } !defined(LONG_DOUBLE) */
				,	(int)i, EQUALS_DOUBLES,	d1, d1
				)
			; print_llo(
				#if (defined(NO_LONG_LONG) || defined(LONG_DOUBLE)) /* { */
					(LONG_ *)&d1
				#else /* } { !defined(NO_LONG_LONG)
				       *	&& !defined(LONG_DOUBLE)
				       */
					AS_LONG_LONG(d1)
				#endif /* } !defined(NO_LONG_LONG)
					*	&& !defined(LONG_DOUBLE)
					*/
					)
			; printf(", "
				#if defined(LONG_DOUBLE) /* { */
					"[%.100Lf][%Le]"
				#else /* } { !defined(LONG_DOUBLE) */
					"[%.100f][%e]"
				#endif /* } !defined(LONG_DOUBLE) */
				,	d2, d2
				)
			; print_llo(
				#if defined(NO_LONG_LONG) || defined(LONG_DOUBLE) /* { */
					(LONG_ *)&d2
				#else /* } { !defined(NO_LONG_LONG)
				       *	&& !defined(LONG_DOUBLE)
				       */
					AS_LONG_LONG(d2)
				#endif /* } !defined(NO_LONG_LONG)
					*	&& !defined(LONG_DOUBLE
					) */
				)
			; printf(">, max_ullps = ")
			;			print_llo(max_ullps)
			; printf(") invokes...\n")
			; (void)fflush(stdout);

			result	= equals_doubles(d1, d2);
			if ((i == 0LL) || result == 0) {
				int	cmp_result;

				printf("[%d] %s("
					#if defined(LONG_DOUBLE) /* { */
						"[%.100Lf][%Le]"
					#else /* } { !defined(LONG_DOUBLE) */
						"[%.100f][%e]"
					#endif /* } !defined(LONG_DOUBLE) */
					, (int)i, EQUALS_DOUBLES, d1, d1
					)
				; print_llo(
					#if (defined(NO_LONG_LONG)	\
					|| defined(LONG_DOUBLE)) /* { */
						(LONG_ *)&d1
					#else /* } { !defined(NO_LONG_LONG)
					       *	&& !defined(LONG_DOUBLE)
					       */
						AS_LONG_LONG(d1)
					#endif /* } !defined(NO_LONG_LONG)
						*	&& !defined(LONG_DOUBLE)
						*/
					)
				; printf(
					#if defined(LONG_DOUBLE) /* { */
						"[%.100Lf][%Le]"
					#else /* } { !defined(LONG_DOUBLE) */
						"[%.100f][%e]"
					#endif /* } !defined(LONG_DOUBLE) */
					,	d2, d2
					)
				; print_llo(
					#if defined(NO_LONG_LONG)	\
					|| defined(LONG_DOUBLE) /* { */
						(LONG_ *)&d2
					#else /* } { !defined(NO_LONG_LONG)
					       *	&& !defined(LONG_DOUBLE)
					       */
						AS_LONG_LONG(d2)
					#endif /* } !defined(NO_LONG_LONG)
						*	&& !defined(LONG_DOUBLE)
						*/
					)
				; printf(">, max_ullps = ")
				;		print_llo(max_ullps)
				; printf(") returns %d.\n", result)
				; (void)fflush(stdout);

				cmp_result	= cmp_double(d1, d2);
				printf("[%d] %s(d1 <"
					#if defined(LONG_DOUBLE) /* { */
						"[%.100Lf][%Le]"
					#else /* } { !defined(LONG_DOUBLE) */
						"[%.100f][%e]"
					#endif /* } !defined(LONG_DOUBLE) */
					, (int)i, CMP_DOUBLE, d1, d1
					)
				; print_llo(
					#if defined(NO_LONG_LONG)	\
					|| defined(LONG_DOUBLE) /* { */
						(LONG_ *)&d1
					#else /* } { !defined(NO_LONG_LONG)
					       *	&& !defined(LONG_DOUBLE)
					       */
						AS_LONG_LONG(d1)
					#endif /* } !defined(NO_LONG_LONG)
						*	&& !defined(LONG_DOUBLE)
						*/
					)
				; printf(">, d2 <"
					#if defined(LONG_DOUBLE) /* { */
						"[%.100Lf][%Le]"
					#else /* } { !defined(LONG_DOUBLE) */
						"[%.100f][%e]"
					#endif /* } !defined(LONG_DOUBLE) */
					,	d2, d2
					)
				; print_llo(
					#if defined(NO_LONG_LONG)	\
					|| defined(LONG_DOUBLE) /* { */
						(LONG_ *)&d2
					#else /* } { !defined(NO_LONG_LONG)
					       *	&& !defined(LONG_DOUBLE)
					       */
						AS_LONG_LONG(d2)
					#endif /* } !defined(NO_LONG_LONG)
						*	&& !defined(LONG_DOUBLE)
						*/
					)
				; printf("> , max_ullps = ")
				;		print_llo(max_ullps)
				; printf(") returns %d.\n", cmp_result)
				; (void)fflush(stdout);

				cmp_result	= cmp_double(d2, d1);
				printf("[%d] %s(d2 <"
					#if defined(LONG_DOUBLE) /* { */
						"[%.100Lf][%Le]"
					#else /* } { !defined(LONG_DOUBLE) */
						"[%.100f][%e]"
					#endif /* } !defined(LONG_DOUBLE) */
					, (int)i, CMP_DOUBLE, d2, d2
					)
				; print_llo(
					#if defined(NO_LONG_LONG)	\
					|| defined(LONG_DOUBLE) /* { */
						(LONG_ *)&d2
					#else /* } { !defined(NO_LONG_LONG)
					       *	&& !defined(LONG_DOUBLE)
					       */
						AS_LONG_LONG(d2)
					#endif /* } !defined(NO_LONG_LONG)
						*	&& !defined(LONG_DOUBLE)
						*/
					)
				; printf("> , d1 <"
					#if defined(LONG_DOUBLE) /* { */
						"[%.100Lf][%Le]"
					#else /* } { !defined(LONG_DOUBLE) */
						"[%.100f][%e]"
					#endif /* } !defined(LONG_DOUBLE) */
					,	d1, d1
					)
				; print_llo(
					#if defined(NO_LONG_LONG)	\
					|| defined(LONG_DOUBLE) /* { */
						(LONG_ *)&d1
					#else /* } { !defined(NO_LONG_LONG)
					       *	&& !defined(LONG_DOUBLE)
					       */
						AS_LONG_LONG(d1)
					#endif /* } !defined(NO_LONG_LONG)
						*	&& !defined(LONG_DOUBLE)
						*/
					)
				; printf("> , max_ullps = ")
				;		print_llo(max_ullps)
				; printf(") returns %d.\n", cmp_result)
				; (void)fflush(stdout);
			}
		}

		printf("<<< UnitTest: %s() and 'max_ullps = "
			,	EQUALS_DOUBLES
			);			print_llo(max_ullps)
		; printf("' finished!\n\n")
		; (void)fflush(stdout);
	}

	/* read 2 doubles and execute 'equals_doubles()'... */
	while (1) {
		printf(">>> ENTER 2 Floating point numbers! "
			"(Quit to type Ctrl-D or Ctrl-C)\n"
			)
		; (void)fflush(stdout);

		(void)fflush(NULL);
		if (fgets(buf1, sizeof(buf1), stdin) == NULL) {
			/* can't read the first numeric string */
			break;
		}
		if (fgets(buf2, sizeof(buf2), stdin) == NULL) {
			/* can't read the second numeric string */
			break; /* stop the process */
		}

		errno	= 0;
		d1	= string2double(chop(buf1));
		printf("d1 = "); print_num(d1)
		; printf(" buf1 = \"%s\" [errno = %d].\n", buf1, errno)
		; (void)fflush(stdout);

		errno	= 0;
		d2	= string2double(chop(buf2));
		printf("d2 = "); print_num(d2)
		; printf(" buf2 = \"%s\" [errno = %d].\n", buf2, errno)
		; (void)fflush(stdout);
		
		errno	= 0;
		result	= equals_doubles(d1, d2);
		printf("equals_doubles(d1, d2) <max_ullps = ")
		;				print_llo(max_ullps)
		; printf("> returns %d [errno = %d].\n", result, errno
		); (void)fflush(stdout);

		errno	= 0;
		result	= cmp_double(d1, d2);
		printf("cmp_doubles(d1, d2) <max_ullps = ")
		;				print_llo(max_ullps)
		; printf("> returns %d [errno = %d].\n", result, errno
		); (void)fflush(stdout);
	}

	exit(0); /* normal exit */
}
#endif /* } defined(UnitTest) */
